import asyncio
import json
import typing

import aiokafka

import carb


class KafkaConsumer:

    def __init__(self, servers: typing.List, topics: typing.List, services: typing.List, group_id: str, app, **kafka_kwargs):
        self._services = services
        self._app = app

        self._servers = servers
        self._topics = topics
        self._group_id = group_id
        self._kafka_kwargs = kafka_kwargs or {}

    async def run(self) -> None:
        kwargs = {
            "bootstrap_servers": self._servers
        }

        if self._group_id:
            kwargs["group_id"] = self._group_id

        kwargs.update(self._kafka_kwargs)

        consumer = aiokafka.AIOKafkaConsumer(
            *self._topics, **kwargs
        )
        await consumer.start()
        try:
            async for msg in consumer:
                for service in self._services:
                    asyncio.ensure_future(self._process_message(msg, service))
        finally:
            await consumer.stop()

    async def _process_message(self, msg, service: str) -> typing.Union[str, bytes]:
        data = json.loads(msg.value)

        data = {
            "topic": msg.topic,
            "partition": msg.partition,
            "offset": msg.offset,
            "key": msg.key.decode("utf-8"),
            "value": data,
            "timestamp": msg.timestamp
        }

        data = json.dumps(data)

        await self._call_service_methods(data, service)

    async def _call_service_methods(self, data: typing.Union[str, bytes], service: str, headers: dict=None) -> None:

        async def _receive():
            try:
                parsed_data = data if isinstance(data, bytes) else bytes(data, "ascii")
                return {"type": "http.request", "body": parsed_data}
            except Exception as exc:
                carb.log_error(f"Error handling body: {exc}")

        scope = {
            "type": "http",  # Change this so it can be amqp etc. as well, will require Router change.
            "method": "POST", # For Kafka we are only assume that POST requests work.
            "path": service,
            "query_string": b"",
            "headers": headers or {},
        }

        async def send(result):
            pass
            # writer.write(bytes(str(result) + "\r\n", "ascii"))
            # await writer.drain()

        try:
            await self._app(scope, _receive, send)
        except Exception as exc:
            carb.log_error(f"Error calling {service}: {exc}")